setfenv(1, MicroMenu.fenv)

local _G, type = _G, type

--[[-----------------------------------------------------------------------------
Local to module
-------------------------------------------------------------------------------]]
local AUTOHIDE_DELAY_TIME, _SetChecked = 1

local function Button_OnClick(self, button, down)
	if not skipClick then
		self = menuData[self.id]
		if self.toggle then
			pcall(_G[self.toggle])
		else
			pcall(ToggleFrame, _G[self.frame])
		end
	end
	UpdateMicroButtons()
	skipClick = nil
end

local function Button_OnDragStart(self, button)
	self.isMoving, self.x, self.y = self:GetFrameStrata(), self:GetCenter()
	self:ClearAllPoints()
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', self.x, self.y)
	self:SetFrameStrata('TOOLTIP')
	self:StartMoving()
end

local function Button_OnDragStop(self, button)
	if not self.isMoving then return end
	self:StopMovingOrSizing()
	self:SetFrameStrata(self.isMoving)
	self.isMoving = nil
	local old, x, y, adj, iMax = self.order, self:GetCenter()
	if x < frame:GetLeft() + buttonSize * (old - 0.5) + settings.spacing * (old - 1) then
		adj, iMax = -1, 1
	else
		adj, iMax = 1, #menuOrder
	end
	local new = iMax
	for index = old + adj, iMax, adj do
		if (buttons[menuOrder[index]]:GetCenter() - x) * adj > 0 then
			new = index - adj
			break
		end
	end
	AdjustMenuOrder(self.id, new)
	return abs(self.x - x) > settings.sensitivity or abs(self.y - y) > settings.sensitivity
end

local function Button_OnMouseDown(self, button)
	GameTooltip:Hide()
	skipLeave = true
	if button == 'LeftButton' and settings.unlockIcons and not IsModifierKeyDown() then
		if Dewdrop:IsOpen(frame) then
			Dewdrop:Close()
		end
		Button_OnDragStart(self, button)
		return
	end
	return Frame_OnMouseDown(frame, button)
end

local function Button_OnMouseUp(self, button)
	if button == 'LeftButton' and Button_OnDragStop(self, button) then
		skipClick, skipLeave = true, true
		return
	end
	return Frame_OnMouseUp(frame, button)
end

local function SetTexCoord(frame, arg, ...)
	local argType = type(arg)
	if argType == 'function' then
		SetTexCoord(frame, arg(...))
	elseif argType == 'table' then
		SetTexCoord(frame, unpack(arg))
	elseif argType == 'string' and type(_G[arg]) ~= 'nil' then
		SetTexCoord(frame, _G[arg], ...)
	elseif argType ~= 'boolean' and argType ~= 'nil' then
		pcall(frame.SetTexCoord, frame, arg, ...)
	else
		frame:SetTexCoord(0, 1, 0, 1)
	end
end

local function SetTexture(frame, arg, ...)
	local argType = type(arg)
	if argType == 'function' then
		SetTexture(frame, arg(...))
	elseif argType == 'table' then
		if arg.texture then
			SetTexture(frame, arg.texture)
			SetTexCoord(frame, arg.texCoord)
		else
			SetTexture(frame, unpack(arg))
		end
	elseif argType == 'string' and type(_G[arg]) ~= 'nil' then
		SetTexture(frame, _G[arg], ...)
	elseif argType ~= 'boolean' and argType ~= 'nil' then
		pcall(frame.SetTexture, frame, arg, ...)
	else
		frame:SetTexture([[Interface\BUTTONS\WHITE8X8]])
		frame:SetVertexColor(0, 0, 0, 1)
	end
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
function Button_OnEnter(self, motion)
	addon:CancelScheduledEvent(addon.title .. "_OnLeave")
	if not self.id then return end
	if Dewdrop:IsOpen(frame) then
		Dewdrop:Close()
		addon:OpenMenu()																				-- Do it this way to prevent menu from closing early
		return
	end
	self = menuData[self.id]
	GameTooltip:SetOwner(frame, 'ANCHOR_BOTTOMLEFT', 0, GetButtonMaxSize() * -3 / 11)
	GameTooltip:SetText(self.title, 1, 1, 1)
	local keyBind = self.keyBind and GetBindingKey(self.keyBind)
	if keyBind then
		GameTooltip:AppendText(NORMAL_FONT_COLOR_CODE .. " (" .. keyBind .. ")")
	end
	if GetCVarBool('showNewbieTips') then
		GameTooltip:AddLine(self.tooltip, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
	end
	if self.tooltipExtend then
		pcall(self.tooltipExtend)
	end
	GameTooltip:Show()
end

function Button_OnLeave(self, motion)
	addon:CancelScheduledEvent(addon.title .. "_OnLeave")
	if GameTooltip:IsOwned(frame) then
		GameTooltip:Hide()
	end
	if Dewdrop:IsOpen(frame) or skipLeave then
		addon:ScheduleEvent(addon.title .. "_OnLeave", Button_OnLeave, AUTOHIDE_DELAY_TIME)
		skipLeave = nil
	end
end

--[[-----------------------------------------------------------------------------
Initialize module
-------------------------------------------------------------------------------]]
frame:SetScript('OnEnter', Button_OnEnter)
frame:SetScript('OnLeave', Button_OnLeave)

for id, data in ipairs(menuData) do
	buttons[id] = CreateFrame('CheckButton', nil, frame)
	buttons[id].id = id
	buttons[id].texture = buttons[id]:CreateTexture(nil, 'ARTWORK')
	buttons[id].texture:SetAllPoints(buttons[id])
	buttons[id]:SetCheckedTexture([[Interface\Buttons\CheckButtonHilight]])
	buttons[id]:SetHighlightTexture([[Interface\Buttons\ButtonHilight-Square]])
	buttons[id]:SetPushedTexture([[Interface\Buttons\ButtonHilight-Square]])
	buttons[id]:GetPushedTexture():SetBlendMode('ADD')

	if data.setIcon then
		buttons[id]:SetScript('OnEvent', data.setIcon)
		buttons[id]:RegisterEvent('PLAYER_ENTERING_WORLD')
	elseif data.texture then
		SetTexture(buttons[id].texture, data.texture)
	elseif data.name then
		buttons[id].texture:SetTexture(format([[Interface\Buttons\UI-MicroButton-%s-Up]], data.name))
	end

	SetTexCoord(buttons[id].texture, data.texCoord)

	if data.background then
		buttons[id].background = buttons[id]:CreateTexture(nil, 'BACKGROUND')
		buttons[id].background:SetAllPoints(buttons[id])
		SetTexture(buttons[id].background, data.background)
	end

	if data.overlay then
		buttons[id].overlay = buttons[id]:CreateTexture(nil, 'OVERLAY')
		buttons[id].overlay:SetAllPoints(buttons[id])
		SetTexture(buttons[id].overlay, data.overlay)
	end

	if data.onUpdate then
		buttons[id]:SetScript('OnUpdate', data.onUpdate)
		buttons[id].updateInterval = 0
	end

	_SetChecked = _SetChecked or buttons[id].SetChecked
	buttons[id].SetChecked = DoNothing
	buttons[id]:SetButtonState('NORMAL', false)
	buttons[id]:SetScript('OnMouseDown', Button_OnMouseDown)
	buttons[id]:SetScript('OnMouseUp', Button_OnMouseUp)
	buttons[id]:SetScript('OnEnter', Button_OnEnter)
	buttons[id]:SetScript('OnLeave', Button_OnLeave)
	buttons[id]:SetScript('OnClick', Button_OnClick)
	buttons[id]:EnableMouse(true)
	buttons[id]:SetMovable(true)
	buttons[id]:Show()
end

---------------------------------------------------------------------------------

hooksecurefunc('HideUIPanel', UpdateMicroButtons)

hooksecurefunc('ShowUIPanel', function(frame)
	local data, id
	frame = frame:GetName()
	for index = 1, #menuOrder do
		id = menuOrder[index]
		data = menuData[id]
		if data.frame == frame then
			_SetChecked(buttons[id], true)
		elseif type(data.frame) == 'table' then
			for index = 1, #data.frames do
				if data.frames[index] == frame then
					return _SetChecked(buttons[id], true)
				end
			end
		end
	end
end)

hooksecurefunc('UpdateMicroButtons', function()
	local checked, data, id
	for index = 1, #menuOrder do
		id = menuOrder[index]
		data = menuData[id]
		checked = _G[data.frame] and _G[data.frame]:IsVisible()
		if not checked and type(data.frames) == 'table' then
			for index = 1, #data.frames do
				checked = _G[data.frames[index]] and _G[data.frames[index]]:IsVisible()
				if checked then break end
			end
		end
		_SetChecked(buttons[id], checked)
	end
end)
