MicroMenu = AceLibrary('AceAddon-2.0'):new('FuBarPlugin-2.0', 'AceDB-2.0', 'AceEvent-2.0')

MicroMenu.fenv = setmetatable({ }, { __index = _G })
setfenv(1, MicroMenu.fenv)

addon = MicroMenu
addon.title = "MicroMenu"
addon.frame = addon:CreateBasicPluginFrame(addon.title .. "_Frame")

buttons, frame, settings = { }, addon.frame, false

addon:RegisterDB("MicroMenuDB")
addon:RegisterDefaults('profile', {
	version		= "",																					-- Don't change
	latency		= 600,
	modifier		= 0,
	scaling		= 1,
	sensitivity	= 2,
	size			= 12,
	sizeOption	= "Automatic",
	spacing		= 3,
	unlockIcons	= false })

MIN_BUTTON_SIZE = 5
MAX_SENSITIVITY = floor(MIN_BUTTON_SIZE / 2)

buttonSize, maxButtonSize = MIN_BUTTON_SIZE

--[[-----------------------------------------------------------------------------
Helper functions
-------------------------------------------------------------------------------]]
function DoNothing()
end

function get_index(name)
	for id = 1, #menuData do
		if menuData[id].name == name then
			return id
		end
	end
end

function table_copy(data)
	if type(data) ~= 'table' then
		return data
	end
	local table = { }
	for key, value in pairs(data) do
		table[key] = table_copy(value)
	end
	return table
end

--[[-----------------------------------------------------------------------------
Internal functions
-------------------------------------------------------------------------------]]
function AdjustMenuOrder(id, new)
	local old = buttons[id].order
	local adj = old > new and -1 or 1
	for index = old, new - adj, adj do
		menuOrder[index] = menuOrder[index + adj]
	end
	menuOrder[new] = id
	CopyMenuOrderToButtons()
	RefreshDisplay()
	return new ~= old
end

function CopyMenuOrderToButtons()
	for id = 1, #buttons do
		buttons[id].order = nil
	end
	for index = 1, #menuOrder do
		buttons[menuOrder[index]].order = index
	end
end

function GetButtonSize()
	return 32
end

function GetButtonMaxSize()
	return GetButtonSize() * 2
end

function RefreshButtonSize()
	buttonSize, maxButtonSize = GetButtonSize(), GetButtonMaxSize()
	if settings.sizeOption == "Automatic" then
		buttonSize = sqrt(maxButtonSize * maxButtonSize * 0.75)
	elseif settings.sizeOption == "Fixed" then
		buttonSize = settings.size
	elseif settings.sizeOption == "Normal" then
		buttonSize = buttonSize + settings.modifier
	else
		buttonSize = sqrt(maxButtonSize * maxButtonSize * settings.scaling)
	end
	buttonSize = max(MIN_BUTTON_SIZE, min(buttonSize, maxButtonSize))
	RefreshDisplay()
end

function RefreshDisplay()
	local button, offsetX
	for id = 1, #buttons do
		button = buttons[id]
		if button.order then
			offsetX = button.order - 1
			button:SetHeight(buttonSize)
			button:SetWidth(buttonSize)
			button:ClearAllPoints()
			button:SetPoint('LEFT', frame, 'LEFT', buttonSize * offsetX + settings.spacing * offsetX, 0)
			button:Show()
		else
			button:Hide()
		end
	end
	frame:SetHeight(maxButtonSize)
	frame:SetWidth(buttonSize * #menuOrder + settings.spacing * (#menuOrder - 1))
end

function ValueCompare(a, b)
	local compare = a - b
	return compare ~= 0 and compare or nil
end

function VersionCompare(a, b)
	local a_major, a_minor, a_revision = VersionSplit(a)
	local b_major, b_minor, b_revision = VersionSplit(b)
	return ValueCompare(a_major, b_major) or ValueCompare(a_minor, b_minor) or ValueCompare(a_revision, b_revision)
end

function VersionSplit(version)
	local major, minor, revision = strsplit(".", version)
	return tonumber(major) or 0, tonumber(minor) or 0, tonumber(revision) or 0
end

--[[-----------------------------------------------------------------------------
Global functions
-------------------------------------------------------------------------------]]
function addon:OnEnable()
	addon:OnProfileEnable()
end

function addon:OnInitialize()
	Frame_OnMouseDown = frame:GetScript('OnMouseDown')										-- Must do this here since FuBar hooks after FuBarPlugin
	Frame_OnMouseUp = frame:GetScript('OnMouseUp')
end

function addon:OnProfileEnable()
	settings = addon.db.profile
	if type(settings.menuOrder) ~= 'table' or settings.menuCount ~= #menuOrderDefault then
		settings.menuCount = #menuOrderDefault
		settings.menuOrder = table_copy(menuOrderDefault)
	end
	if VersionCompare(settings.version, addon.version) then								-- Handle any issues with different version's saved variables
		settings.hidden = nil
		settings.sensitivity = settings.sensitivity > MAX_SENSITIVITY and MAX_SENSITIVITY or settings.sensitivity
		settings.version = addon.version
	end
	LATENCY_MID_RANGE = settings.latency / 2
	menuOrder = settings.menuOrder
	CopyMenuOrderToButtons()
	RefreshButtonSize()
end
