setfenv(1, MicroMenu.fenv)

local max, min = max, min

local GetNetStats, PERFORMANCEBAR_UPDATE_INTERVAL = GetNetStats, PERFORMANCEBAR_UPDATE_INTERVAL or 10

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local default_texCoord, mainMenu_texCoord = { 5/32, 27/32, 31/64, 58/64 }, { 4/32, 28/32, 33/64, 53/64 }

local function Character_UpdateIcon()
	SetPortraitTexture(buttons[get_index('Character')].texture, 'player')
end

local latency
local function MainMenu_ExtendTooltip()
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(format(MAINMENUBAR_LATENCY_LABEL, latency or select(3, GetNetStats())), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	GameTooltip:AddLine(format(MAINMENUBAR_FPS_LABEL, GetFramerate()), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
end

local timer = 0
local function MainMenu_OnUpdate(self, elapsed)
	timer = timer - elapsed
	if timer > 0 then return end
	timer = PERFORMANCEBAR_UPDATE_INTERVAL
	_, _, latency = GetNetStats()
	self.overlay:SetVertexColor(min(latency / LATENCY_MID_RANGE, 1), 1 - min(max(latency - LATENCY_MID_RANGE, 0) / LATENCY_MID_RANGE, 1), 0)
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
menuData = {
	{
		background = true,
		frame = 'CharacterFrame',
		keyBind = 'TOGGLECHARACTER0',
		name = 'Character',
		setIcon = Character_UpdateIcon,
		texCoord = { 0.1, 0.9, 0.0666, 0.9 },
		title = CHARACTER_BUTTON,
		tooltip = NEWBIE_TOOLTIP_CHARACTER
	},

	{
		frame = 'SpellBookFrame',
		name = 'Spellbook',
		keyBind = 'TOGGLESPELLBOOK',
		texCoord = default_texCoord,
		title = SPELLBOOK_ABILITIES_BUTTON,
		tooltip = NEWBIE_TOOLTIP_SPELLBOOK
	},

	{
		frame = 'PlayerTalentFrame',
		keyBind = 'TOGGLETALENTS',
		name = 'Talents',
		texCoord = default_texCoord,
		title = TALENTS_BUTTON,
		toggle = 'ToggleTalentFrame',
		tooltip = NEWBIE_TOOLTIP_TALENTS
	},

	{
		frame = 'AchievementFrame',
		keyBind = 'TOGGLEACHIEVEMENT',
		name = 'Achievement',
		texCoord = default_texCoord,
		title = ACHIEVEMENT_BUTTON,
		toggle = 'ToggleAchievementFrame',
		tooltip = NEWBIE_TOOLTIP_ACHIEVEMENT
	},

	{
		frame = 'QuestLogFrame',
		keyBind = 'TOGGLEQUESTLOG',
		name = 'Quest',
		texCoord = default_texCoord,
		title = QUESTLOG_BUTTON,
		tooltip = NEWBIE_TOOLTIP_QUESTLOG
	},

	{
		frame = 'FriendsFrame',
		keyBind = 'TOGGLESOCIAL',
		name = 'Socials',
		texCoord = default_texCoord,
		title = SOCIAL_BUTTON,
		tooltip = NEWBIE_TOOLTIP_SOCIAL
	},

	{
		background = true,
		frame = 'PVPParentFrame',
		keyBind = 'TOGGLECHARACTER4', --'TOGGLEPVP',
		name = 'PVP',
		setIcon = function(self) self.texture:SetTexture([[Interface\TargetingFrame\UI-PVP-]] .. UnitFactionGroup('player')) end,
		texCoord = { 4/64, 38/64, 2/64, 38/64 },
		title = PLAYER_V_PLAYER,
		tooltip = NEWBIE_TOOLTIP_PVP
	},

	{
		frame = 'LFDParentFrame',
		keyBind = 'TOGGLELFGPARENT',
		name = 'LFG',
		texCoord = default_texCoord,
		title = DUNGEONS_BUTTON,
		tooltip =  NEWBIE_TOOLTIP_LFGPARENT
	},

	{
		frame = 'GameMenuFrame',
		frames = { 'AudioOptionsFrame', 'InterfaceOptionsFrame', 'KeyBindingFrame', 'MacroFrame', 'VideoOptionsFrame' },
		keyBind = 'TOGGLEGAMEMENU',
		name = 'MainMenu',
		onUpdate = MainMenu_OnUpdate,
		overlay = { texture = format([[Interface\AddOns\FuBar_%s\LatencyOverlay]], addon.title), texCoord = mainMenu_texCoord },
		texCoord = mainMenu_texCoord,
		texture = format([[Interface\AddOns\FuBar_%s\MainMenu]], addon.title),
		title = MAINMENU_BUTTON,
		tooltip = NEWBIE_TOOLTIP_MAINMENU,
		tooltipExtend = MainMenu_ExtendTooltip
	},

	{
		frame = 'HelpFrame',
		name = 'Help',
		texCoord = default_texCoord,
		title = HELP_BUTTON,
		toggle = 'ToggleHelpFrame',
		tooltip =  NEWBIE_TOOLTIP_HELP
	}
}

--[[-----------------------------------------------------------------------------
Initialize module
-------------------------------------------------------------------------------]]
addon:RegisterEvent('UNIT_PORTRAIT_UPDATE', Character_UpdateIcon)

menuOrderDefault = { }
for id = 1, #menuData do
	menuOrderDefault[id] = id
end
menuOrder = menuOrderDefault																		-- In case referenced in a hooked function before established
