setfenv(1, MicroMenu.fenv)

Dewdrop = AceLibrary('Dewdrop-2.0')

MAX_BUTTON_MODIFIER	= 25
MAX_BUTTON_SIZE		= 50
MAX_BUTTON_SPACING	= 25
MIN_BUTTON_MODIFIER	= -25

--[[-----------------------------------------------------------------------------
Local to module
-------------------------------------------------------------------------------]]
local function GetValue(field)
	return settings[field]
end

local function SetValue(field, value)
	settings[field] = value
end

local function SetValueWithRefresh(field, value)
	settings[field] = value
	RefreshButtonSize()
end

local function GetButtonStatus(id)
	return buttons[id].order
end

local function SetButtonStatus(id, value)
	if not value then
		for index = 1, #menuOrder do
			if menuOrder[index] == id then
				tremove(menuOrder, index)
				break
			end
		end
		id = #menuOrder < 1 and 1 or nil
	end
	if id then
		menuOrder[#menuOrder + 1] = id
	end
	CopyMenuOrderToButtons()
	RefreshDisplay()
end

local function HideLatencyOption()
	return not buttons[get_index('MainMenu')].order
end

local function SetLatencyRange(k, v)
	settings[k] = v
	LATENCY_MID_RANGE = v / 2
	local _, _, latency = GetNetStats()
	buttons[get_index('MainMenu')].overlay:SetVertexColor(min(latency / LATENCY_MID_RANGE, 1), 1 - min(max(latency - LATENCY_MID_RANGE, 0) / LATENCY_MID_RANGE, 1), 0)
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
settingsMenu = { name = "Options", type = 'group', desc = format("%s's options.", addon.title), args = {

	{ name = "Appearance", type = 'group', order = 1, desc = format("Options to change the size and look of %s in the FuBar panel.", addon.title), args = {

		{ type = 'header', order = 100 },

		{ name = "Latency Indicator", type = 'range', order = 101, passValue = "latency", max = 1000, min = 100, step = 10,
		  desc = "The computer screen on the Game Menu button changes color to indicate latency. This value determines the point at which it will be red.",
		hidden = HideLatencyOption,
		   set = SetLatencyRange,
		   get = GetValue },

		{ type = 'header', order = 102, hidden = HideLatencyOption },

		{ name = "Size Options", type = 'text', order = 103, passValue = "sizeOption",
		  desc = format("Choose how the size of the buttons will be determined.  The size cannot exceed the bounds of the FuBar panel or be smaller than %s x %s.", MIN_BUTTON_SIZE, MIN_BUTTON_SIZE),
		   set = SetValueWithRefresh,
		   get = GetValue,
  	validate = { "Automatic", "Fixed", "Normal", "Scale" },
 	validateDesc = { ["Automatic"] = "Size is determined automatically based on FuBar's settings.",
						  ["Fixed"] = "Size is set to a fixed value.",
						  ["Normal"] = "Size will match FuBar's font size and may be modified further by a fixed amount.",
						  ["Scale"] = "Size will be scaled relative to the maximum as determined by FuBar's settings." } },

		{ name = "Set Size", type = 'range', order = 104, passValue = "size", max = MAX_BUTTON_SIZE, min = MIN_BUTTON_SIZE, step = 1,
		  desc = "Set a value to use for the size of the buttons.",
		hidden = function() return settings.sizeOption ~= "Fixed" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Set Modifier", type = 'range', order = 105, passValue = "modifier", max = MAX_BUTTON_MODIFIER, min = MIN_BUTTON_MODIFIER, step = 1,
		  desc = "Set the amount to modify FuBar's font size by.",
		hidden = function() return settings.sizeOption ~= "Normal" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Set Scaling", type = 'range', order = 106, passValue = "scaling", max = 1, min = 0, step = 0.01, isPercent = true,
		  desc = "Set the amount to scale the size relative to the maximum.",
		hidden = function() return settings.sizeOption ~= "Scale" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Spacing", type = 'range', order = 107, passValue = "spacing", max = MAX_BUTTON_SPACING, min = 0, step = 1,
		  desc = "Set the spacing between the buttons.",
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ type = 'header', order = 108 },

		{ name = "Sensitivity", type = 'range', order = 109, passValue = "sensitivity", max = MAX_SENSITIVITY, min = 0, step = 1,
		  desc = "Set the amount you can move an unlocked button and still have it count as a click.",
		   set = SetValue,
		   get = GetValue },

		{ name = "Unlock", type = 'toggle', order = 110, passValue = "unlockIcons",
		  desc = "Unlock the buttons to change their order.",
		   set = SetValue,
		   get = GetValue } } } } }

for id, data in pairs(menuData) do
	tinsert(settingsMenu.args[1].args, {
		name = data.title or data.name, type = 'toggle', order = id, passValue = id,
		desc = format("Show the %s button.", data.title or data.name),
		set = SetButtonStatus,
		get = GetButtonStatus
	})
end

--[[-----------------------------------------------------------------------------
Global
-------------------------------------------------------------------------------]]
function addon:OnMenuRequest()
	Dewdrop:FeedAceOptionsTable(settingsMenu)
end

--[[-----------------------------------------------------------------------------
Initialize module
-------------------------------------------------------------------------------]]
Dewdrop:InjectAceOptionsTable(addon, settingsMenu)
